@@ -873,20 +873,22 @@ tcp_srv_connected (GObject *source,
       else
         g_clear_error (&error);
 
-      DEBUG ("Falling back to HOST connection");
-
       priv->state = WCON_TCP_CONNECTING;
-
       /* decode a hostname from the JID here: Don't check for an explicit *
        * connect host supplied by the user as we shouldn't even try a SRV *
        * connection in that case, and should therefore never get here     */
       wocky_decode_jid (priv->jid, &node, &host, NULL);
 
       if ((host != NULL) && (*host != '\0'))
-        connect_to_host_async (connector, host, port);
+        {
+          DEBUG ("Falling back to HOST connection to %s", host);
+          connect_to_host_async (connector, host, port);
+        }
       else
-        abort_connect_code (self, WOCKY_CONNECTOR_ERROR_BAD_JID,
-            "JID contains no domain: %s", priv->jid);
+        {
+          abort_connect_code (self, WOCKY_CONNECTOR_ERROR_BAD_JID,
+              "JID contains no domain: %s", priv->jid);
+        }
 
       g_free (node);
       g_free (host);
@@ -921,11 +923,12 @@ tcp_host_connected (GObject *source,
           DEBUG ("we previously hit a GIOError when connecting using SRV; "
               "reporting that error");
           abort_connect_error (connector, &priv->srv_connect_error,
-              "Bad SRV record");
+              "couldn't connect to server specified by SRV record");
         }
       else
         {
-          abort_connect_error (connector, &error, "connection failed");
+          abort_connect_error (connector, &error,
+              "couldn't connect to server");
         }
 
       g_error_free (error);
@@ -1032,7 +1035,7 @@ maybe_old_ssl (WockyConnector *self)
 
       DEBUG ("Beginning SSL handshake");
       wocky_tls_connector_secure_async (tls_connector,
-          priv->conn, TRUE, get_peername (self),
+          priv->conn, TRUE, get_peername (self), NULL,
           priv->cancellable, tls_connector_secure_cb, self);
 
       g_object_unref (tls_connector);
@@ -1105,7 +1108,8 @@ xmpp_init_recv_cb (GObject *source,
   priv->session_id = g_strdup (id);
 
   debug = state_message (priv, "");
-  DEBUG ("%s: received XMPP v%s stream open from server", debug, version);
+  DEBUG ("%s: received XMPP version=%s stream open from server", debug,
+      version != NULL ? version : "(unspecified)");
   g_free (debug);
 
   ver = (version != NULL) ? atof (version) : -1;
@@ -1117,12 +1121,13 @@ xmpp_init_recv_cb (GObject *source,
             "Server not XMPP 1.0 Compliant");
       else
         jabber_request_auth (self);
-      goto out;
     }
-
-  DEBUG ("waiting for feature stanza from server");
-  wocky_xmpp_connection_recv_stanza_async (priv->conn, priv->cancellable,
-      xmpp_features_cb, data);
+  else
+    {
+      DEBUG ("waiting for feature stanza from server");
+      wocky_xmpp_connection_recv_stanza_async (priv->conn, priv->cancellable,
+          xmpp_features_cb, data);
+    }
 
  out:
   g_free (version);
@@ -1223,7 +1228,7 @@ xmpp_features_cb (GObject *source,
 
       tls_connector = wocky_tls_connector_new (priv->tls_handler);
       wocky_tls_connector_secure_async (tls_connector,
-          priv->conn, FALSE, get_peername (self), priv->cancellable,
+          priv->conn, FALSE, get_peername (self), NULL, priv->cancellable,
           tls_connector_secure_cb, self);
 
       g_object_unref (tls_connector);
