@@ -550,10 +550,21 @@ _new_connection (GSocketService *service,
     gpointer user_data)
 {
   WockyMetaPorter *self = user_data;
-  GSocketAddress *addr = g_socket_connection_get_remote_address (
-      socket_connection, NULL);
+  GSocketAddress *addr;
   GInetAddress *inet_address;
   gchar *str;
+  GError *error = NULL;
+
+  addr = g_socket_connection_get_remote_address (
+      socket_connection, &error);
+
+  if (addr == NULL)
+    {
+      DEBUG ("New connection, but failed to get remote address "
+          "so ignoring: %s", error->message);
+      g_clear_error (&error);
+      return FALSE;
+    }
 
   addr = normalize_address (addr);
 
@@ -562,7 +573,7 @@ _new_connection (GSocketService *service,
 
   str = g_inet_address_to_string (inet_address);
 
-  DEBUG ("new connection from %s!", str);
+  DEBUG ("New connection from %s!", str);
 
   wocky_ll_connector_incoming_async (G_IO_STREAM (socket_connection),
       NULL, new_connection_connect_cb, g_object_ref (self));
@@ -680,6 +691,9 @@ wocky_meta_porter_constructed (GObject *obj)
   WockyMetaPorter *self = WOCKY_META_PORTER (obj);
   WockyMetaPorterPrivate *priv = self->priv;
 
+  if (G_OBJECT_CLASS (wocky_meta_porter_parent_class)->constructed)
+    G_OBJECT_CLASS (wocky_meta_porter_parent_class)->constructed (obj);
+
   priv->listener = g_socket_service_new ();
   g_signal_connect (priv->listener, "incoming",
       G_CALLBACK (_new_connection), self);
