@@ -55,17 +55,23 @@ void gibber_debug_stanza (DebugFlags flag, WockyStanza *stanza,
 
 #ifdef DEBUG_FLAG
 
-#define DEBUG(format, ...) \
-  G_STMT_START { } G_STMT_END
+static inline void
+DEBUG (
+    const gchar *format,
+    ...)
+{
+}
 
-#define DEBUG_STANZA(stanza, format, ...) \
-  G_STMT_START { } G_STMT_END
+static inline void
+DEBUG_STANZA (
+    WockyStanza *stanza,
+    const gchar *format,
+    ...)
+{
+}
 
 #define DEBUGGING 0
 
-#define NODE_DEBUG(n, s) \
-  G_STMT_START { } G_STMT_END
-
 #endif /* DEBUG_FLAG */
 
 #endif /* ENABLE_DEBUG */
