@@ -29,22 +29,20 @@
 #include "gibber-signals-marshal.h"
 
 #include "gibber-sockets.h"
-#include "gibber-namespaces.h"
-#include "gibber-xmpp-reader.h"
-#include "gibber-xmpp-writer.h"
 #include "gibber-multicast-transport.h"
 #include "gibber-r-multicast-transport.h"
 #include "gibber-r-multicast-causal-transport.h"
 
+#include <wocky/wocky-xmpp-reader.h>
+#include <wocky/wocky-xmpp-writer.h>
+#include <wocky/wocky-namespaces.h>
+
 #define ADDRESS_KEY "address"
 #define PORT_KEY "port"
 
 #define DEBUG_FLAG DEBUG_MUC_CONNECTION
 #include "gibber-debug.h"
 
-static void _reader_received_stanza_cb (GibberXmppReader *reader,
-    WockyStanza *stanza, gpointer user_data);
-
 static void _connection_received_data (GibberTransport *transport,
     GibberBuffer *buffer, gpointer user_data);
 
@@ -79,8 +77,8 @@ struct _GibberMucConnectionPrivate
   gchar *address;
   gchar *port;
 
-  GibberXmppReader *reader;
-  GibberXmppWriter *writer;
+  WockyXmppReader *reader;
+  WockyXmppWriter *writer;
 
   GHashTable *parameters;
 
@@ -88,7 +86,6 @@ struct _GibberMucConnectionPrivate
   GibberRMulticastCausalTransport *rmctransport;
   GibberRMulticastTransport *rmtransport;
 
-  const gchar *current_sender;
   GArray *streams_used;
   guint16 last_stream_allocated;
   gulong rmc_connected_handler;
@@ -114,10 +111,8 @@ gibber_muc_connection_init (GibberMucConnection *obj)
   guint16 stream_id;
 
   /* allocate any data required by the object here */
-  priv->reader = gibber_xmpp_reader_new_no_stream ();
-  priv->writer = gibber_xmpp_writer_new_no_stream ();
-  g_signal_connect (priv->reader, "received-stanza",
-                   G_CALLBACK (_reader_received_stanza_cb), obj);
+  priv->reader = wocky_xmpp_reader_new_no_stream ();
+  priv->writer = wocky_xmpp_writer_new_no_stream ();
 
   priv->streams_used = g_array_sized_new (FALSE, TRUE, sizeof (guint16), 1);
   /* 0 is the "default" stream */
@@ -274,7 +269,7 @@ gibber_muc_connection_finalize (GObject *object)
 const gchar **
 gibber_muc_connection_get_protocols (void)
 {
-  static const gchar *protocols[] = { GIBBER_TELEPATHY_NS_CLIQUE, NULL };
+  static const gchar *protocols[] = { WOCKY_TELEPATHY_NS_CLIQUE, NULL };
   return protocols;
 }
 
@@ -288,7 +283,7 @@ gibber_muc_connection_get_required_parameters (const gchar *protocol)
   struct {
     const gchar *protocol;
     const gchar **parameters;
-  } protocols[] = { { GIBBER_TELEPATHY_NS_CLIQUE, parameters },
+  } protocols[] = { { WOCKY_TELEPATHY_NS_CLIQUE, parameters },
                     { NULL, NULL }
                   };
 
@@ -394,7 +389,7 @@ gibber_muc_connection_new (const gchar *name, const gchar *protocol,
   GibberMucConnection *result;
   GibberMucConnectionPrivate *priv;
 
-  if (protocol != NULL && strcmp (protocol, GIBBER_TELEPATHY_NS_CLIQUE) != 0)
+  if (protocol != NULL && strcmp (protocol, WOCKY_TELEPATHY_NS_CLIQUE) != 0)
     {
       g_set_error (error, GIBBER_MUC_CONNECTION_ERROR,
           GIBBER_MUC_CONNECTION_ERROR_INVALID_PROTOCOL,
@@ -431,7 +426,7 @@ gibber_muc_connection_new (const gchar *name, const gchar *protocol,
     }
   else
     {
-      priv->protocol = g_strdup (GIBBER_TELEPATHY_NS_CLIQUE);
+      priv->protocol = g_strdup (WOCKY_TELEPATHY_NS_CLIQUE);
     }
 
   priv->address = g_strdup (address);
@@ -630,19 +625,6 @@ gibber_muc_connection_get_parameters (GibberMucConnection *connection)
   return priv->parameters;
 }
 
-static void
-_reader_received_stanza_cb (GibberXmppReader *reader, WockyStanza *stanza,
-    gpointer user_data)
-{
-  GibberMucConnection *self = GIBBER_MUC_CONNECTION (user_data);
-  GibberMucConnectionPrivate *priv = GIBBER_MUC_CONNECTION_GET_PRIVATE (self);
-
-  g_assert (priv->current_sender != NULL);
-  g_signal_emit (self, signals[RECEIVED_STANZA], 0,
-      priv->current_sender, stanza);
-}
-
-
 static void
 _connection_received_data (GibberTransport *transport, GibberBuffer *buffer,
     gpointer user_data)
@@ -650,7 +632,7 @@ _connection_received_data (GibberTransport *transport, GibberBuffer *buffer,
   GibberMucConnection *self = GIBBER_MUC_CONNECTION (user_data);
   GibberMucConnectionPrivate *priv = GIBBER_MUC_CONNECTION_GET_PRIVATE (self);
   GibberRMulticastBuffer *rmbuffer = (GibberRMulticastBuffer *) buffer;
-  gboolean ret;
+  WockyStanza *stanza;
   GError *error = NULL;
 
   g_assert (buffer->length > 0);
@@ -663,19 +645,31 @@ _connection_received_data (GibberTransport *transport, GibberBuffer *buffer,
       return;
     }
 
-  /* Ensure we're not disposed inside while running the reader is busy */
-  g_object_ref (self);
-  priv->current_sender = rmbuffer->sender;
-  ret = gibber_xmpp_reader_push (priv->reader, buffer->data, buffer->length,
-      &error);
-  priv->current_sender = NULL;
+  /* push the data into the reader */
+  wocky_xmpp_reader_push (priv->reader, buffer->data, buffer->length);
 
-  if (!ret)
+  error = wocky_xmpp_reader_get_error (priv->reader);
+
+  if (error != NULL)
     {
+      DEBUG ("reader error: %s", error->message);
       g_signal_emit (self, signals[PARSE_ERROR], 0);
+      g_clear_error (&error);
+
+      wocky_xmpp_reader_reset (priv->reader);
     }
 
-  g_object_unref (self);
+  /* now check if we got a stanza out of it */
+  stanza = wocky_xmpp_reader_pop_stanza (priv->reader);
+
+  if (stanza != NULL)
+    {
+      g_signal_emit (self, signals[RECEIVED_STANZA], 0,
+          rmbuffer->sender, stanza);
+      g_object_unref (stanza);
+
+      wocky_xmpp_reader_reset (priv->reader);
+    }
 }
 
 gboolean
@@ -687,11 +681,8 @@ gibber_muc_connection_send (GibberMucConnection *connection,
   const guint8 *data;
   gsize length;
 
-  if (!gibber_xmpp_writer_write_stanza (priv->writer, stanza,
-      &data, &length, error))
-    {
-      return FALSE;
-    }
+  wocky_xmpp_writer_write_stanza (priv->writer, stanza,
+      &data, &length);
 
   return gibber_transport_send (GIBBER_TRANSPORT (priv->rmtransport),
       data, length, error);
