@@ -35,7 +35,7 @@
 #include <telepathy-glib/svc-generic.h>
 
 #include <gibber/gibber-linklocal-transport.h>
-#include <gibber/gibber-namespaces.h>
+#include <wocky/wocky-namespaces.h>
 #include <wocky/wocky-stanza.h>
 #include <wocky/wocky-meta-porter.h>
 
@@ -250,8 +250,11 @@ salut_im_channel_set_property (GObject *object,
         priv->connection = g_value_get_object (value);
         break;
       case PROP_HANDLE_TYPE:
-        g_assert (g_value_get_uint (value) == 0 ||
-            g_value_get_uint (value) == TP_HANDLE_TYPE_CONTACT);
+        /* 0 is the old tp-glib value of unset, TP_UNKNOWN_HANDLE_TYPE is the
+         * new version */
+        g_assert (g_value_get_uint (value) == 0
+            || g_value_get_uint (value) == TP_HANDLE_TYPE_CONTACT
+            || g_value_get_uint (value) == TP_UNKNOWN_HANDLE_TYPE);
         break;
       case PROP_CHANNEL_TYPE:
         tmp = g_value_get_string (value);
@@ -534,7 +537,7 @@ new_message_cb (WockyPorter *porter,
   SalutImChannel *self = SALUT_IM_CHANNEL (user_data);
 
   if (wocky_node_get_child_ns (wocky_stanza_get_top_node (stanza), "invite",
-        GIBBER_TELEPATHY_NS_CLIQUE) != NULL)
+        WOCKY_TELEPATHY_NS_CLIQUE) != NULL)
     /* we don't handle Clique MUC invites here, so pass it on to the
      * next handler */
     return FALSE;
