@@ -33,8 +33,7 @@
 
 #include "muc-channel.h"
 
-#include <gibber/gibber-namespaces.h>
-#include <gibber/gibber-transport.h>
+#include <wocky/wocky-namespaces.h>
 
 #include <telepathy-glib/channel-iface.h>
 #include <telepathy-glib/interfaces.h>
@@ -44,6 +43,7 @@
 #include <telepathy-glib/util.h>
 
 #include <gibber/gibber-muc-connection.h>
+#include <gibber/gibber-transport.h>
 
 #include "connection.h"
 #include "contact-manager.h"
@@ -268,8 +268,11 @@ salut_muc_channel_set_property (GObject     *object,
       priv->muc_connection = g_value_get_object (value);
       break;
     case PROP_HANDLE_TYPE:
+        /* 0 is the old tp-glib value of unset, TP_UNKNOWN_HANDLE_TYPE is the
+         * new version */
       g_assert (g_value_get_uint (value) == 0
-               || g_value_get_uint (value) == TP_HANDLE_TYPE_ROOM);
+               || g_value_get_uint (value) == TP_HANDLE_TYPE_ROOM
+               || g_value_get_uint (value) == TP_UNKNOWN_HANDLE_TYPE);
       break;
     case PROP_CHANNEL_TYPE:
       tmp = g_value_get_string (value);
@@ -484,7 +487,7 @@ create_invitation (SalutMucChannel *self, SalutContact *contact,
       ')',
       '(', "invite",
         '*', &invite_node,
-        ':', GIBBER_TELEPATHY_NS_CLIQUE,
+        ':', WOCKY_TELEPATHY_NS_CLIQUE,
         '(', "roomname",
           '$', tp_handle_inspect (room_repo, priv->handle),
         ')',
@@ -612,9 +615,7 @@ salut_muc_channel_add_member (GObject *iface,
                               GError **error)
 {
   SalutMucChannel *self = SALUT_MUC_CHANNEL(iface);
-#ifdef ENABLE_DEBUG
   SalutMucChannelPrivate *priv = SALUT_MUC_CHANNEL_GET_PRIVATE (self);
-#endif
   TpBaseConnection *base_connection = TP_BASE_CONNECTION (self->connection);
   TpIntSet *empty, *remote_pending;
 
@@ -660,7 +661,7 @@ salut_muc_channel_add_member (GObject *iface,
 
   /* Adding a contact, let's invite him */
 
-  DEBUG ("Trying to add handle %u to %s", handle ,priv->object_path);
+  DEBUG ("Trying to add handle %u to %s", handle, priv->object_path);
 
   if (!salut_muc_channel_send_invitation (self, handle, message, error))
     return FALSE;
@@ -944,13 +945,11 @@ salut_muc_channel_invited (SalutMucChannel *self, TpHandle inviter,
                           const gchar *stanza, GError **error)
 {
   TpBaseConnection *base_connection = TP_BASE_CONNECTION (self->connection);
-#ifdef ENABLE_DEBUG
   SalutMucChannelPrivate *priv = SALUT_MUC_CHANNEL_GET_PRIVATE (self);
   TpHandleRepoIface *contact_repo =
       tp_base_connection_get_handles (base_connection, TP_HANDLE_TYPE_CONTACT);
   TpHandleRepoIface *room_repo =
       tp_base_connection_get_handles (base_connection, TP_HANDLE_TYPE_ROOM);
-#endif
   gboolean ret = TRUE;
 
   /* Got invited to this muc channel */
@@ -1135,7 +1134,7 @@ salut_muc_channel_received_stanza (GibberMucConnection *conn,
 #endif
 
   tubes_node = wocky_node_get_child_ns (node, "tubes",
-      GIBBER_TELEPATHY_NS_TUBES);
+      WOCKY_TELEPATHY_NS_TUBES);
   if (tubes_node != NULL)
     {
       SalutTubesChannel *tubes_chan;
