@@ -38,7 +38,6 @@
 #include <telepathy-glib/svc-channel.h>
 #include <telepathy-glib/svc-generic.h>
 
-#include <gibber/gibber-bytestream-ibb.h>
 #include <gibber/gibber-bytestream-muc.h>
 #include <gibber/gibber-muc-connection.h>
 
@@ -1360,7 +1359,7 @@ data_received_cb (GibberBytestreamIface *stream,
   sender = tp_handle_lookup (contact_repo, from, NULL, NULL);
   if (sender == 0)
     {
-      DEBUG ("unkown sender: %s", from);
+      DEBUG ("unknown sender: %s", from);
       return;
     }
 
