@@ -1,8 +1,7 @@
 /*
- * util.h - Headers for Gabble utility functions
- * Copyright (C) 2006 Collabora Ltd.
+ * util.h - Headers for Salut utility functions
+ * Copyright (C) 2006-2007 Collabora Ltd.
  * Copyright (C) 2006 Nokia Corporation
- *   @author Robert McQueen <robert.mcqueen@collabora.co.uk>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,10 +18,18 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <glib.h>
-
 #ifndef __SALUT_UTIL_H__
 #define __SALUT_UTIL_H__
 
-gpointer mixin_offset_cast (gpointer instance, guint offset);
+#include <glib.h>
+#include <glib-object.h>
+#include <wocky/wocky-stanza.h>
+
+/* Mapping a XMPP node with a GHashTable */
+GHashTable *salut_wocky_node_extract_properties (WockyNode *node,
+    const gchar *prop);
+void salut_wocky_node_add_children_from_properties (WockyNode *node,
+    GHashTable *properties, const gchar *prop);
+gchar *salut_generate_id (void);
+
 #endif /* __SALUT_UTIL_H__ */
